/**************************************************************************** 
*  Copyright 2007 C&H Technologies, Inc.  All Rights Reserved.					    * 
*****************************************************************************/
/****************************************************************************
*
*	ma203_ansi.h
*
*	Header file for the MA203 ANSI-C source module.
*
*	C&H Part Number: 11028109
*
*****************************************************************************/
#ifndef MA203_HEADER
#define	MA203_HEADER

#ifndef		PLATFORM
#define		PLATFORM chwin
#endif

#include	"apis.h"


/**************************************************************************** 
*  TypeDef's
*****************************************************************************/
typedef char Int8;
typedef unsigned char UInt8;
typedef unsigned short UInt16;
typedef short Int16;
typedef int Int32;
typedef unsigned int UInt32;
typedef unsigned short Bool;
typedef double Real64;

typedef struct
{	APIS_HANDLE apis_handle;
	Int32				apis_error;
}MA203_INSTR, *MA203_HANDLE;


/**************************************************************************** 
*  Error Codes
*****************************************************************************/
#define		MA203_SUCCESS															0

#define		MA203_SPECIFIC_ERROR_BASE									0xBFFC0900

#define		MA203_ERROR_RESERVED_VALUE								(MA203_SPECIFIC_ERROR_BASE + 0)
#define		MA203_ERROR_DEBOUNCE_SELECT_MISMATCH			(MA203_SPECIFIC_ERROR_BASE + 1)
#define		MA203_ERROR_RUNNING               				(MA203_SPECIFIC_ERROR_BASE + 2)
#define		MA203_ERROR_INVALID_ACCESSORY_FUNCTION		(MA203_SPECIFIC_ERROR_BASE + 3)
#define		MA203_ERROR_PARAMETER1										(MA203_SPECIFIC_ERROR_BASE + 4)
#define		MA203_ERROR_PARAMETER2										(MA203_SPECIFIC_ERROR_BASE + 5)
#define		MA203_ERROR_PARAMETER3										(MA203_SPECIFIC_ERROR_BASE + 6)
#define		MA203_ERROR_PARAMETER4										(MA203_SPECIFIC_ERROR_BASE + 7)
#define		MA203_ERROR_FAIL_ID_QUERY									(MA203_SPECIFIC_ERROR_BASE + 8)
#define		MA203_ERROR_APIS													(MA203_SPECIFIC_ERROR_BASE + 9)
#define		MA203_ERROR_OUT_OF_MEMORY									(MA203_SPECIFIC_ERROR_BASE + 10)	
#define		MA203_ERROR_INVALID_DEBOUNCE_TIME 				(MA203_SPECIFIC_ERROR_BASE + 11)
#define		MA203_ERROR_FIFO_EMPTY										(MA203_SPECIFIC_ERROR_BASE + 12)
#define		MA203_SPECIFIC_ERROR_END									(MA203_SPECIFIC_ERROR_BASE + 13)
#define		MA203_SPECIFIC_ERROR_ABSOLUTE_MAX					(MA203_SPECIFIC_ERROR_BASE + 0x6FF)

#define		MA203_TRUE		1
#define		MA203_FALSE		0


/**************************************************************************** 
*  Warning Codes
*****************************************************************************/
#define		MA203_SPECIFIC_WARNING_BASE													0x3FFC0900
#define		MA203_SPECIFIC_WARNING_NUM_SAMPLES_NOT_AVAILABLE			(MA203_SPECIFIC_WARNING_BASE + 0)
#define		MA203_SPECIFIC_WARNING_END													(MA203_SPECIFIC_WARNING_BASE + 1)
#define		MA203_SPECIFIC_WARNING_ABSOLUTE_MAX									(MA203_SPECIFIC_WARNING_BASE + 0x6FF)


/**************************************************************************** 
*  M-Module ID Words
*****************************************************************************/
#define		MA203_IDENT_WORD_SYNC_CODE				0
#define		MA203_IDENT_WORD_MODULE_NUM				1
#define		MA203_IDENT_WORD_REVISION_NUM 		2
#define		MA203_IDENT_WORD_MODULE_CHAR 			3
#define		MA203_IDENT_WORD_VXI_SYNC_CODE 		16
#define		MA203_IDENT_WORD_VXI_ID 					17
#define		MA203_IDENT_WORD_VXI_DEVICE_TYPE  18

#define		MA203_VXI_MANF_ID				0xFC1
#define		MA203_VXI_MODEL_CODE		0xFE8

/**************************************************************************** 
*  Run Source Defined Values
*****************************************************************************/
#define		MA203_RUN_SOURCE_SOFTWARE_BIT			0
#define		MA203_RUN_SOURCE_EXTRUN_HIGH			2
#define		MA203_RUN_SOURCE_EXTRUN_LOW				3
#define		MA203_RUN_SOURCE_TRIGA_HIGH				4
#define		MA203_RUN_SOURCE_TRIGA_LOW				5
#define		MA203_RUN_SOURCE_TRIGB_HIGH				6
#define		MA203_RUN_SOURCE_TRIGB_LOW				7


/**************************************************************************** 
*	Debounce value channel groups
*****************************************************************************/
#define		MA203_CHANNEL_GROUP_0_3       0
#define		MA203_CHANNEL_GROUP_4_7       1
#define		MA203_CHANNEL_GROUP_8_12      2
#define		MA203_CHANNEL_GROUP_13_15     3
#define		MA203_CHANNEL_GROUP_ALL				4


/**************************************************************************** 
*	MA203 Register offsets
*****************************************************************************/
#define		MA203_CONTROL_STATUS_REGISTER						0x00
#define		MA203_CLOCK_CONTROL_REGISTER						0x02
#define		MA203_MASTER_INTERRUPT_REGISTER					0x04
#define		MA203_DEBOUNCE_CONTROL_REGISTER					0x06
#define		MA203_POLARITY_REGISTER									0x08
#define		MA203_WATCH_REGISTER										0x0A
#define		MA203_CHANNEL_INTERRUPT_REGISTER				0x0C
#define		MA203_INTERRUPT_DEFINITION_REGISTER			0x0E
#define		MA203_INTERRUPT_PENDING_REGISTER				0x10
#define		MA203_FIFO_DATA_REGISTER								0x12
#define		MA203_CURRENT_VALUE_REGISTER						0x14
#define		MA203_LAST_VALUE_STORED_REGISTER				0x16
#define		MA203_FIFO_UNREAD_COUNT_REGISTER				0x18
#define		MA203_ACCESSORY_MODULE_REGISTER					0x1A
#define		MA203_ID_ACCESS_REGISTER								0xFE


/**************************************************************************** 
*	Bit Definitions
*****************************************************************************/
#define		MA203_DEBOUNCE_CONTROL_BIT							0x0800
#define		MA203_RESET_BIT													0x0001
#define		MA203_RUN_SEL_BITS  										0x0070
#define		MA203_STORE_ALL_BIT											0x0080
#define		MA203_INTERRUPT_TYPE_BIT								0x0400
#define		MA203_MASTER_INTERRUPT_ENABLE_BIT  			0x0200
#define		MA203_RUN_BIT														0x0001
#define		MA203_SINGLE_SAMPLE_BIT									0x0008
#define		MA203_DATA_VALID_BIT										0x8000
#define		MA203_RESET_FIFO_BIT										0x0004
#define		MA203_DATA_STORED_BIT										0x8000
#define		MA203_FIFO_HALF_FULL_BIT								0x2000
#define		MA203_FIFO_FULL_BIT											0x4000
#define		MA203_RESET_TIME_STAMP_BIT							0x0002
#define		MA203_TIME_STAMP_ROLLOVER_BIT						0x1000
#define		MA203_STATUS_BITS												0xF000

/**************************************************************************** 
*	Accessory Module Definitions
*****************************************************************************/
#define		MA203_ACC_OPTION_4_RESOLUTION		(25.5/255.0)
#define		MA203_ACC_OPTION_5_RESOLUTION		(32.0/255.0)
#define		MA203_ACC_OPTION_5_OFFSET				-2.0
#define		MA203_ACC_OPTION_6_RESOLUTION		(40.0/4095.0)





/*************************************************************************** 
*	Exported Function Prototypes
*****************************************************************************/



/*  Low level Access  */
Int32 ma203_ReadRegister (MA203_HANDLE handle, UInt16 offset, UInt16 *value);
Int32 ma203_WriteRegister (MA203_HANDLE handle, UInt16 offset, UInt16 value);
Int32 ma203_GetIdentWord (MA203_HANDLE handle, UInt16 offset, UInt16 *value);


/*  Configuration  */
Int32 ma203_SetPolarity (MA203_HANDLE handle, UInt16 polarityMask);
Int32 ma203_GetPolarity (MA203_HANDLE handle, UInt16 *polarityMask);
Int32 ma203_SetWatchMask (MA203_HANDLE handle, UInt16 watchMask);
Int32 ma203_GetWatchMask (MA203_HANDLE handle, UInt16 *watchMask);
Int32 ma203_ConfigureDebounce (MA203_HANDLE handle, Real64 channles03, Real64 channles47, Real64 channles811,
                                             Real64 channles1215);
Int32 ma203_ConfigureSampleClock (MA203_HANDLE handle, Int32 clockSource, Int32 internalClockRate, 
                                                Int32 clockPrescaler,
                                                Bool clockOutSelect,
                                                Bool clockOutEnable,
                                                Bool triggerOutSelect);
Int32 ma203_ConfigureRunSource (MA203_HANDLE handle, Int32 runSource);
Int32 ma203_StoreAll (MA203_HANDLE handle, Bool storeAll);



/*  interrupt functions  */
Int32 ma203_EnableInterrupts (MA203_HANDLE handle, Bool interruptEnable, UInt8 interruptVector);
Int32 ma203_InstallIrq (MA203_HANDLE handle, void *interruptHandler, Int32 vector, Int32 level, Int32 mode, void *var_ptr);
Int32 ma203_UninstallIrq (MA203_HANDLE handle);
Int32 ma203_ConfigIntConditions (MA203_HANDLE handle, Bool dataStored, Bool fifoFull, Bool fifoHalfFull,
                                               Bool timeStampRollover,
                                               Bool pattern,
                                               UInt16 interruptDefinition,
                                               UInt16 channelInterruptEnable);
Int32 ma203_ClearInterrupts (MA203_HANDLE handle, UInt16 clearInterruptMask);
Int32 ma203_GetInterruptPending (MA203_HANDLE handle, UInt16 *interruptPendingMask);
Int32 ma203_ReEnableInterrupts (MA203_HANDLE handle);

/*  Accessory Module Options  */
Int32 ma203_AccessoryModuleOption (MA203_HANDLE handle, Int32 *optionNumber, Int8 *optionDescription);
Int32 ma203_SetInputThreshold (MA203_HANDLE handle, Real64 inputThreshold, UInt16 channelMask);

/*  Perform Sampling  */
Int32 ma203_Run (MA203_HANDLE handle);
Int32 ma203_Stop (MA203_HANDLE handle);
Int32 ma203_SingleSample (MA203_HANDLE handle);
Int32 ma203_CurrentValue (MA203_HANDLE handle, UInt16 *currentValue);
Int32 ma203_LastStoredValue (MA203_HANDLE handle, UInt16 *lastStoredValue);

/*  Fifo  */
Int32 ma203_ReadFifo (MA203_HANDLE handle, UInt32 *timeStamp, UInt16 *value);
Int32 ma203_ReadFifoBuffer (MA203_HANDLE handle, UInt32 count, UInt32 *time_stamp_buffer, UInt16 *data_buffer, UInt32 *num_read);
Int32 ma203_ResetFifo (MA203_HANDLE handle);
Int32 ma203_FifoUnreadCount (MA203_HANDLE handle, UInt16 *unreadCount);
Int32 ma203_GetFifoStatus (MA203_HANDLE handle, Bool *fifoFull, Bool *fifoHalfFull, Bool *dataStored);

/*  Time Stamp  */
Int32 ma203_ResetTimeStamp (MA203_HANDLE handle);
Int32 ma203_GetTimeStampStatus (MA203_HANDLE handle, Bool *timeStampRollover);
Int32 ma203_ClearTimeStampStatus (MA203_HANDLE handle);

/* Utility Functions  */
Int32 ma203_RevisionQuery (Int8 driver_rev[]);
Int32 ma203_ErrorMessage (Int32 statusCode, Int8 errorMessage[]);


/*****************************************************************************
*	Modify the functions below this point to use the MA203 driver in an 
*	environment not supported by an APIS platform support module. All functions
*	above this point should not have to be modified.
*
*	In general, any call to an APIS function (apis_func()) will need to be
*	modified to communicate with the instrument or operating system being used.
*
******************************************************************************/
Int32 ma203_Init (Int32 instr, Bool IDQuery, Bool reset, MA203_HANDLE *handle);
Int32 ma203_Close (MA203_HANDLE handle);
Int32 ma203_Reset (MA203_HANDLE handle);


Int32 ma203_GetApisError (MA203_HANDLE handle, UInt32 *apis_error);



#endif

         

